// Trade Crossings - by RF_Gandalf
// A Random Map for Age of Empires III Demo

include "mercenaries.xs";

// Main entry point for random map script
void main(void)
{
   // Text
   rmSetStatusText("",0.01);

   // Picks the map size
   int playerTiles=15500;
   if (cNumberNonGaiaPlayers >4)
	playerTiles = 11500;
   if (cNumberNonGaiaPlayers >6)
      playerTiles = 10500;

   int size=2.0*sqrt(cNumberNonGaiaPlayers*playerTiles);
   rmEchoInfo("Map size="+size+"m x "+size+"m");
   rmSetMapSize(size, size);

   // Some map turbulence...
   rmSetMapElevationParameters(cElevTurbulence, 0.4, 5, 0.7, 5.0);  
   rmSetMapElevationHeightBlend(1);

   // Picks a default map type and height
   rmSetSeaLevel(0.0);
   rmTerrainInitialize("newengland_grass");
   rmSetMapType("newEngland");
   rmSetMapType("land");
   rmSetWorldCircleConstraint(true);
   rmSetWindMagnitude(2.0);
   rmSetLightingSet("new england");
   rmSetMapType("grass");

   // Choose mercs.
   chooseMercs();

   // Define some classes. These are used later for constraints.
   int classPlayer=rmDefineClass("player");
   int classPlayerCore=rmDefineClass("player core");
   rmDefineClass("center");
   rmDefineClass("classCliff");
   rmDefineClass("classPatch");
   rmDefineClass("corner");
   rmDefineClass("starting settlement");
   rmDefineClass("startingUnit");
   rmDefineClass("classForest");
   rmDefineClass("importantItem");
   rmDefineClass("secrets");
   rmDefineClass("socketClass");
   rmDefineClass("nuggets");

   // Text
   rmSetStatusText("",0.10);

   // CONSTRAINTS
   
   // Map edge constraints
   int edgeConstraint=rmCreateBoxConstraint("edge of map", rmXTilesToFraction(20), rmZTilesToFraction(20), 1.0-rmXTilesToFraction(20), 1.0-rmZTilesToFraction(20), 0.01);
   int playerEdgeConstraint=rmCreateBoxConstraint("player edge of map", rmXTilesToFraction(6), rmZTilesToFraction(6), 1.0-rmXTilesToFraction(6), 1.0-rmZTilesToFraction(6), 0.01);
   int longPlayerConstraint=rmCreateClassDistanceConstraint("stays away from players", classPlayer, 16.0);

   // Player constraints
   int playerConstraint=rmCreateClassDistanceConstraint("player vs. player", classPlayer, 10.0);
   int smallMapPlayerConstraint=rmCreateClassDistanceConstraint("stay away from players a lot", classPlayer, 70.0);

   // Center constraint.
   int centerConstraint=rmCreateClassDistanceConstraint("stay away from center", rmClassID("center"), 25.0);
   int wideCenterConstraint=rmCreateClassDistanceConstraint("elevation avoids center", rmClassID("center"), 50.0);

   // Resource avoidance
   int forestConstraint=rmCreateClassDistanceConstraint("forest vs. forest", rmClassID("classForest"), 10.0);
   int avoidDeer=rmCreateTypeDistanceConstraint("food avoids food", "deer", 45.0);
   int avoidFastCoin=rmCreateTypeDistanceConstraint("fast coin avoids coin", "gold", 30.0);
   int avoidCoin=rmCreateTypeDistanceConstraint("coin avoids coin", "gold", 35);
   int avoidNugget=rmCreateTypeDistanceConstraint("nugget avoid nugget", "AbstractNugget", 35.0);
   int avoidNuggetSmall=rmCreateTypeDistanceConstraint("nugget avoid nugget small", "AbstractNugget", 10.0);

   // Avoid impassable land
   int avoidImpassableLand=rmCreateTerrainDistanceConstraint("avoid impassable land", "Land", false, 6.0);
   int shortAvoidImpassableLand=rmCreateTerrainDistanceConstraint("short avoid impassable land", "Land", false, 2.0);
   int avoidCliffs=rmCreateClassDistanceConstraint("cliff vs. cliff", rmClassID("classCliff"), 30.0);
   int patchConstraint=rmCreateClassDistanceConstraint("patch vs. patch", rmClassID("classPatch"), 5.0);
   int wallConstraint=rmCreateClassDistanceConstraint("wall vs. wall", rmClassID("classWall"), 40.0);
   int avoidSheep=rmCreateTypeDistanceConstraint("sheep avoids sheep", "sheep", 40.0);

   // Specify true so constraint stays outside of circle (i.e. inside corners)
   int cornerConstraint0=rmCreateCornerConstraint("in corner 0", 0, true);
   int cornerConstraint1=rmCreateCornerConstraint("in corner 1", 1, true);
   int cornerConstraint2=rmCreateCornerConstraint("in corner 2", 2, true);
   int cornerConstraint3=rmCreateCornerConstraint("in corner 3", 3, true);

   // Unit avoidance
   int avoidStartingUnits=rmCreateClassDistanceConstraint("objects avoid starting units", rmClassID("startingUnit"), 20.0);
   int avoidStartingUnitsSmall=rmCreateClassDistanceConstraint("objects avoid starting units small", rmClassID("startingUnit"), 5.0);

   // ships vs. ships
   int shipVsShip=rmCreateTypeDistanceConstraint("ships avoid ship", "ship", 5.0);

   // Decoration avoidance
   int avoidAll=rmCreateTypeDistanceConstraint("avoid all", "all", 6.0);

   // VP avoidance
   int avoidTradeRoute = rmCreateTradeRouteDistanceConstraint("trade route", 10.0);
   int avoidImportantItem=rmCreateClassDistanceConstraint("secrets etc avoid each other", rmClassID("importantItem"), 30.0);
   int avoidSocket=rmCreateClassDistanceConstraint("avoid sockets", rmClassID("socketClass"), 10.0);

   int circleConstraint=rmCreatePieConstraint("circle Constraint", 0.5, 0.5, 0, rmZFractionToMeters(0.47), rmDegreesToRadians(0), rmDegreesToRadians(360));

   int whaleLand = rmCreateTerrainDistanceConstraint("whale v. land", "land", true, 20.0);

   // Text
   rmSetStatusText("",0.20);

   // PLAYER PLACEMENT
   int playerSetup =  rmRandInt(1,2);

   if(cNumberNonGaiaPlayers == 2)
   {
      if (playerSetup == 1)
         rmPlacePlayersLine(0.2, 0.2, 0.8, 0.8, 0, 0);
      else
         rmPlacePlayersLine(0.22, 0.78, 0.78, 0.22, 0, 0); 
   }
   else if(cNumberTeams == 2)
   {
      if (playerSetup == 1)      
         rmPlacePlayersLine(0.18, 0.18, 0.82, 0.82, 0, 20); 
      else
         rmPlacePlayersLine(0.18, 0.82, 0.82, 0.18, 0, 20); 
   }
   else
      rmPlacePlayersCircular(0.35, 0.4, rmDegreesToRadians(5.0));

   // Text
   rmSetStatusText("",0.30);

   // PLAYER AREAS
   float playerFraction=rmAreaTilesToFraction(280);
   for(i=1; <cNumberPlayers)
   {
      // Create the area.
      int id=rmCreateArea("Player"+i);
      // Assign to the player.
      rmSetPlayerArea(i, id);
      // Set the size.
      rmSetAreaSize(id, playerFraction, playerFraction);
      rmAddAreaToClass(id, classPlayer);
      rmSetAreaMinBlobs(id, 3);
      rmSetAreaMaxBlobs(id, 4);
      rmSetAreaMinBlobDistance(id, 30.0);
      rmSetAreaMaxBlobDistance(id, 50.0);
      rmSetAreaSmoothDistance(id, 20);
      rmAddAreaConstraint(id, playerConstraint); 
      rmSetAreaLocPlayer(id, i);
      rmSetAreaWarnFailure(id, false);
   }

   // Text
   rmSetStatusText("",0.4);

   // Placement order
   // Trade route -> Lakes -> Natives -> Secrets -> Cliffs -> Nuggets

   // TRADE ROUTES
   // First route - SE to NW
   int routeType = rmRandInt(1,4);
   int tradeRouteID = rmCreateTradeRoute();
   if (routeType == 4)
   {
	rmAddTradeRouteWaypoint(tradeRouteID, 0.5, 1.0);
	rmAddTradeRouteWaypoint(tradeRouteID, 0.45, 0.70);
	rmAddTradeRouteWaypoint(tradeRouteID, 0.5, 0.5);
	rmAddTradeRouteWaypoint(tradeRouteID, 0.55, 0.30);
	rmAddTradeRouteWaypoint(tradeRouteID, 0.5, 0.0);
   }
   else if(routeType == 3)
   {
      rmAddTradeRouteWaypoint(tradeRouteID, 0.5, 1.0);
      rmAddTradeRouteWaypoint(tradeRouteID, 0.55, 0.70);
      rmAddTradeRouteWaypoint(tradeRouteID, 0.5, 0.5);
      rmAddTradeRouteWaypoint(tradeRouteID, 0.45, 0.30);
      rmAddTradeRouteWaypoint(tradeRouteID, 0.5, 0.0);
   }
   else if(routeType == 2)
   {
	rmAddTradeRouteWaypoint(tradeRouteID, 0.3, 1.0);
	rmAddTradeRouteWaypoint(tradeRouteID, 0.47, 0.73);
	rmAddTradeRouteWaypoint(tradeRouteID, 0.5, 0.5);
	rmAddTradeRouteWaypoint(tradeRouteID, 0.53, 0.27);
	rmAddTradeRouteWaypoint(tradeRouteID, 0.7, 0.0);

   }
   else if(routeType == 1)
   {
	rmAddTradeRouteWaypoint(tradeRouteID, 0.6, 1.0);
	rmAddTradeRouteWaypoint(tradeRouteID, 0.48, 0.70);
	rmAddTradeRouteWaypoint(tradeRouteID, 0.5, 0.5);
	rmAddTradeRouteWaypoint(tradeRouteID, 0.52, 0.30);
	rmAddTradeRouteWaypoint(tradeRouteID, 0.4, 0.0);
   }  

   bool placedTradeRoute = rmBuildTradeRoute(tradeRouteID, "dirt");
   if(placedTradeRoute == false)
		rmEchoError("Failed to place trade route #1");

   // Second route - SW to NE
   routeType = rmRandInt(1,4);
   int tradeRouteID2 = rmCreateTradeRoute();
   if (routeType == 4)
   {
	rmAddTradeRouteWaypoint(tradeRouteID2, 0.0, 0.5);
	rmAddTradeRouteWaypoint(tradeRouteID2, 0.30, 0.55);
	rmAddTradeRouteWaypoint(tradeRouteID2, 0.5, 0.5);
	rmAddTradeRouteWaypoint(tradeRouteID2, 0.70, 0.45);
	rmAddTradeRouteWaypoint(tradeRouteID2, 1.0, 0.5);
   }
   else if(routeType == 3)
   {
      rmAddTradeRouteWaypoint(tradeRouteID2, 0.0, 0.5);
      rmAddTradeRouteWaypoint(tradeRouteID2, 0.30, 0.55);
      rmAddTradeRouteWaypoint(tradeRouteID2, 0.5, 0.5);
      rmAddTradeRouteWaypoint(tradeRouteID2, 0.70, 0.45);
      rmAddTradeRouteWaypoint(tradeRouteID2, 1.0, 0.5);
   }
   else if(routeType == 2)
   {
	rmAddTradeRouteWaypoint(tradeRouteID2, 0.0, 0.7);
	rmAddTradeRouteWaypoint(tradeRouteID2, 0.25, 0.53);
	rmAddTradeRouteWaypoint(tradeRouteID2, 0.5, 0.5);
	rmAddTradeRouteWaypoint(tradeRouteID2, 0.75, 0.47);
	rmAddTradeRouteWaypoint(tradeRouteID2, 1.0, 0.3);
   }
   else if(routeType == 1) 
   {  
	rmAddTradeRouteWaypoint(tradeRouteID2, 0.0, 0.4);
	rmAddTradeRouteWaypoint(tradeRouteID2, 0.30, 0.48);
	rmAddTradeRouteWaypoint(tradeRouteID2, 0.5, 0.5);
	rmAddTradeRouteWaypoint(tradeRouteID2, 0.70, 0.48);
	rmAddTradeRouteWaypoint(tradeRouteID2, 1.0, 0.6);
   }

   bool placedTradeRoute2 = rmBuildTradeRoute(tradeRouteID2, "carolinas\trade_route");
   if(placedTradeRoute2 == false)
		rmEchoError("Failed to place trade route #2");

   // Trade sockets
   int socketID=rmCreateObjectDef("sockets to dock Trade Posts");
   rmAddObjectDefItem(socketID, "SocketTradeRoute", 1, 0.0);
   rmSetObjectDefAllowOverlap(socketID, true);
   rmAddObjectDefToClass(socketID, rmClassID("importantItem"));
   rmSetObjectDefMinDistance(socketID, 0.0);
   rmSetObjectDefMaxDistance(socketID, 6.0);

   // For first Trade Route.
   float socketNum = rmRandFloat(0.15, 0.31);
   rmSetObjectDefTradeRouteID(socketID, tradeRouteID);
   vector socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, socketNum);
   rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
   socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, (1.00-socketNum));
   rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);

   // For second Trade Route
   float socketNum2 = rmRandFloat(0.15, 0.31);
   rmSetObjectDefTradeRouteID(socketID, tradeRouteID2);
   socketLoc = rmGetTradeRouteWayPoint(tradeRouteID2, (1.00-socketNum2));
   rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);

   socketLoc = rmGetTradeRouteWayPoint(tradeRouteID2, socketNum2);
   rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);

   // Text
   rmSetStatusText("",0.50);

   // NATIVE AMERICANS

   int whichNative=rmRandInt(1,3);
     
   int villageAID = -1;
   int villageType = rmRandInt(1,3);
   if ( whichNative == 1 )
      {
	villageAID = rmCreateGrouping("iroquois village A", "native iroquois village "+villageType);
      }
   else if ( whichNative == 2 )
      {
	villageAID = rmCreateGrouping("cherokee village A", "native cherokee village "+villageType);
      }
   else 
      {
	villageAID = rmCreateGrouping("comanche village A", "native comanche village "+villageType);
      }
   rmSetGroupingMinDistance(villageAID, 0.0);
   rmSetGroupingMaxDistance(villageAID, rmXFractionToMeters(0.05));
   rmAddGroupingConstraint(villageAID, avoidImpassableLand);
   rmAddGroupingToClass(villageAID, rmClassID("importantItem"));
   rmAddGroupingConstraint(villageAID, avoidTradeRoute);
   if (playerSetup == 1)
      rmPlaceGroupingAtLoc(villageAID, 0, 0.78, 0.22); 
   else
      rmPlaceGroupingAtLoc(villageAID, 0, 0.2, 0.2);
	
   int villageBID = -1;
   villageType = rmRandInt(1,3);
   if ( whichNative == 1 )
	{
	villageBID = rmCreateGrouping("iroquois village B", "native iroquois village "+villageType);
	}
   else if ( whichNative == 2 )
	{
	villageBID = rmCreateGrouping("cherokee village B", "native cherokee village "+villageType);
	}
   else
	{
	villageBID = rmCreateGrouping("comanche village B", "native comanche village "+villageType);
	}
   rmSetGroupingMinDistance(villageBID, 0.0);
   rmSetGroupingMaxDistance(villageBID, rmXFractionToMeters(0.05));
   rmAddGroupingConstraint(villageBID, avoidImpassableLand);
   rmAddGroupingToClass(villageBID, rmClassID("importantItem"));
   rmAddGroupingConstraint(villageBID, avoidTradeRoute);
   if (playerSetup == 1)
      rmPlaceGroupingAtLoc(villageBID, 0, 0.22, 0.78);
   else
      rmPlaceGroupingAtLoc(villageBID, 0, 0.8, 0.8);

   // STARTING UNITS
   int startingUnits = rmCreateStartingUnitsObjectDef(5.0);
   rmSetObjectDefMinDistance(startingUnits, 5.0);
   rmSetObjectDefMaxDistance(startingUnits, 10.0);
   rmAddObjectDefToClass(startingUnits, rmClassID("startingUnit"));
   rmAddGroupingToClass(startingUnits, rmClassID("importantItem"));
   rmAddObjectDefConstraint(startingUnits, avoidAll);

   int startingTCID= rmCreateObjectDef("startingTC");
   if ( rmGetNomadStart())
	{
	rmAddObjectDefItem(startingTCID, "CoveredWagon", 1, 0.0);
	}
   else
	{
	rmAddObjectDefItem(startingTCID, "TownCenter", 1, 0.0);
	}
   rmAddObjectDefToClass(startingTCID, rmClassID("startingUnit"));

   int silverType = -1;
   int playerGoldID = -1;
   int playerGold2ID = -1;

   int StartAreaTreeID=rmCreateObjectDef("starting trees");
   rmAddObjectDefItem(StartAreaTreeID, "TreeNewEngland", 1, 0.0);
   rmSetObjectDefMinDistance(StartAreaTreeID, 10.0);
   rmSetObjectDefMaxDistance(StartAreaTreeID, 15.0);
   rmAddObjectDefConstraint(StartAreaTreeID, avoidStartingUnitsSmall);

   int StartBerryBushID=rmCreateObjectDef("starting BerryBush");
   rmAddObjectDefItem(StartBerryBushID, "BerryBush", 3, 5.0);
   rmSetObjectDefMinDistance(StartBerryBushID, 10.0);
   rmSetObjectDefMaxDistance(StartBerryBushID, 15.0);
   rmAddObjectDefConstraint(StartBerryBushID, avoidStartingUnitsSmall);

   int playerNuggetID=rmCreateObjectDef("player nugget");
   rmAddObjectDefItem(playerNuggetID, "nugget", 1, 0.0);
   rmAddObjectDefToClass(playerNuggetID, rmClassID("nuggets"));
   rmSetObjectDefMinDistance(playerNuggetID, 30.0);
   rmSetObjectDefMaxDistance(playerNuggetID, 35.0);
   rmAddObjectDefConstraint(playerNuggetID, avoidStartingUnitsSmall);
   rmAddObjectDefConstraint(playerNuggetID, avoidNugget);
   rmAddObjectDefConstraint(playerNuggetID, circleConstraint);

   for(i=1; <cNumberPlayers)
   {
	// Place starting units and a TC!
	rmPlaceObjectDefAtLoc(startingTCID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
	rmPlaceObjectDefAtLoc(startingUnits, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));

	// Everyone gets one ore grouping close by and a second not far off.
	silverType = rmRandInt(1,10);
	playerGoldID = rmCreateObjectDef("player silver closer "+i);
	rmAddObjectDefItem(playerGoldID, "mine", 1, 0.0);
      rmAddGroupingToClass(playerGoldID, rmClassID("importantItem"));
	rmAddObjectDefConstraint(playerGoldID, avoidTradeRoute);
	rmAddObjectDefConstraint(playerGoldID, avoidStartingUnitsSmall);
	rmSetObjectDefMinDistance(playerGoldID, 15.0);
	rmSetObjectDefMaxDistance(playerGoldID, 20.0);
	rmPlaceObjectDefAtLoc(playerGoldID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));

	silverType = rmRandInt(1,10);
	playerGold2ID = rmCreateObjectDef("player silver second "+i);
	rmAddObjectDefItem(playerGold2ID, "mine", 1, 0.0);
	rmAddObjectDefConstraint(playerGold2ID, avoidTradeRoute);
	rmAddObjectDefConstraint(playerGold2ID, avoidStartingUnitsSmall);
	rmSetObjectDefMinDistance(playerGold2ID, 38.0);
	rmSetObjectDefMaxDistance(playerGold2ID, 45.0);
	rmPlaceObjectDefAtLoc(playerGold2ID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));

	// Placing starting Pronghorns...
	rmPlaceObjectDefAtLoc(StartBerryBushID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
		
	// Placing starting trees...
	rmPlaceObjectDefAtLoc(StartAreaTreeID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
	rmPlaceObjectDefAtLoc(StartAreaTreeID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
	rmPlaceObjectDefAtLoc(StartAreaTreeID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
	rmPlaceObjectDefAtLoc(StartAreaTreeID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
	rmPlaceObjectDefAtLoc(StartAreaTreeID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));

	// Nuggets
	rmSetNuggetDifficulty(1, 1);
	rmPlaceObjectDefAtLoc(playerNuggetID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
	rmPlaceObjectDefAtLoc(playerNuggetID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));

   }

   // CLIFFS
   int numCliffs = rmRandInt(3,5);
   for (i=0; <numCliffs)
   {    
		int bigCliffID=rmCreateArea("big cliff" +i);
	      rmSetAreaSize(bigCliffID, rmAreaTilesToFraction(160), rmAreaTilesToFraction(250));
		rmSetAreaWarnFailure(bigCliffID, false);
		rmSetAreaCliffType(bigCliffID, "New England");
		rmAddAreaToClass(bigCliffID, rmClassID("classCliff"));	
		rmSetAreaCliffEdge(bigCliffID, 1, 0.6, 0.1, 1.0, 0);
		rmSetAreaCliffPainting(bigCliffID, false, true, true, 1.5, true);
		rmSetAreaCliffHeight(bigCliffID, 7, 1.0, 1.0);
		rmSetAreaHeightBlend(bigCliffID, 2.0);
//		rmAddAreaTerrainLayer(bigCliffID, "new_england\ground4_ne", 0, 1);
		rmSetAreaMinBlobs(bigCliffID, 8);
		rmSetAreaMaxBlobs(bigCliffID, 10);
		rmSetAreaMinBlobDistance(bigCliffID, 12.0);
		rmSetAreaMaxBlobDistance(bigCliffID, 20.0);
		rmSetAreaSmoothDistance(bigCliffID, 10);
		rmSetAreaCoherence(bigCliffID, 0.5);
		rmAddAreaConstraint(bigCliffID, avoidImportantItem);
		rmAddAreaConstraint(bigCliffID, avoidTradeRoute);
	      rmAddAreaConstraint(bigCliffID, avoidCliffs);
	      rmAddAreaConstraint(bigCliffID, avoidSocket);
	      rmAddAreaConstraint(bigCliffID, edgeConstraint);
	      rmAddAreaConstraint(bigCliffID, smallMapPlayerConstraint);
	      rmAddAreaConstraint(bigCliffID, longPlayerConstraint);
		rmAddAreaInfluenceSegment(bigCliffID, 0.6, 0.52, 0.65, 0.68);
		rmBuildArea(bigCliffID);
   }

   // Text
   rmSetStatusText("",0.60);

   // NUGGETS
   int nuggetID= rmCreateObjectDef("nugget"); 
   rmAddObjectDefItem(nuggetID, "Nugget", 1, 0.0);
   rmSetObjectDefMinDistance(nuggetID, 0.0);
   rmSetObjectDefMaxDistance(nuggetID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(nuggetID, shortAvoidImpassableLand);
   rmAddObjectDefConstraint(nuggetID, avoidNugget);
   rmAddObjectDefConstraint(nuggetID, avoidStartingUnits);
   rmAddObjectDefConstraint(nuggetID, avoidTradeRoute);
   rmAddObjectDefConstraint(nuggetID, avoidSocket);
   rmAddObjectDefConstraint(nuggetID, avoidAll);
   rmAddObjectDefConstraint(nuggetID, circleConstraint);
   rmSetNuggetDifficulty(2, 3);
   rmPlaceObjectDefPerPlayer(nuggetID, false, 5); 

    // Text
   rmSetStatusText("",0.65); 
 
   // MINES
   int silverID = -1;
   int silverCount = cNumberNonGaiaPlayers*2;	// 2 per player, plus starting two.
   rmEchoInfo("silver count = "+silverCount);

   for(i=0; < silverCount)
   {
	silverType = rmRandInt(1,10);
	silverID = rmCreateObjectDef("silver "+i);
	rmAddObjectDefItem(silverID, "mine", 1, 0.0);
	rmSetObjectDefMinDistance(silverID, 0.0);
	rmSetObjectDefMaxDistance(silverID, rmXFractionToMeters(0.5));

	rmAddObjectDefConstraint(silverID, avoidFastCoin);
	rmAddObjectDefConstraint(silverID, avoidCoin);
	rmAddObjectDefConstraint(silverID, avoidAll);
	rmAddObjectDefConstraint(silverID, avoidImpassableLand);
	rmAddObjectDefConstraint(silverID, avoidTradeRoute);
	rmAddObjectDefConstraint(silverID, avoidSocket);
	rmAddObjectDefConstraint(silverID, avoidStartingUnits);
	// Keep silver away from the water, to avoid the art problem with the "cliffs."
	rmPlaceObjectDefAtLoc(silverID, 0, 0.5, 0.5);
   }

   // FORESTS
   int forestTreeID = 0;
   int numTries=11*cNumberNonGaiaPlayers;
   int failCount=0;

   for (i=0; <numTries)
   {   
      int forest=rmCreateArea("forest "+i);
      rmSetAreaWarnFailure(forest, false);
      rmSetAreaSize(forest, rmAreaTilesToFraction(90), rmAreaTilesToFraction(150));
      rmSetAreaForestType(forest, "new england forest");
      rmSetAreaForestDensity(forest, 1.0);
      rmSetAreaForestClumpiness(forest, 0.9);
      rmSetAreaForestUnderbrush(forest, 0.0);
      rmSetAreaCoherence(forest, 0.4);
      rmSetAreaSmoothDistance(forest, 10);
      rmAddAreaToClass(forest, rmClassID("classForest")); 
      rmAddAreaConstraint(forest, forestConstraint);
      rmAddAreaConstraint(forest, avoidAll);
      rmAddAreaConstraint(forest, avoidImpassableLand); 
      rmAddAreaConstraint(forest, avoidTradeRoute);
	rmAddAreaConstraint(forest, avoidStartingUnits);
	rmAddAreaConstraint(forest, avoidSocket);
      if(rmBuildArea(forest)==false)
      {
         // Stop trying once we fail 3 times in a row.
         failCount++;
         if(failCount==5)
            break;
      }
      else
         failCount=0; 
   } 
 
   // Text
   rmSetStatusText("",0.70);

   // DEER
   int playerDeerID=rmCreateObjectDef("player's deer herd");
   int bonusChance=rmRandFloat(0, 1);
   if(bonusChance<0.25)
      rmAddObjectDefItem(playerDeerID, "deer", 6, 5.0);
   else if(bonusChance<0.5)
      rmAddObjectDefItem(playerDeerID, "deer", 7, 6.0);
   else if(bonusChance<0.75)
      rmAddObjectDefItem(playerDeerID, "deer", 8, 6.0);
   else
      rmAddObjectDefItem(playerDeerID, "deer", 9, 6.0);
   rmSetObjectDefMinDistance(playerDeerID, 15.0);
   rmSetObjectDefMaxDistance(playerDeerID, 25.0);
   rmAddObjectDefConstraint(playerDeerID, avoidDeer);
   rmAddObjectDefConstraint(playerDeerID, avoidAll);
   rmAddObjectDefConstraint(playerDeerID, avoidSocket);
   rmAddObjectDefConstraint(playerDeerID, avoidImpassableLand);
   rmAddObjectDefConstraint(playerDeerID, avoidStartingUnits);
   rmSetObjectDefCreateHerd(playerDeerID, true);
   rmPlaceObjectDefPerPlayer(playerDeerID, false, 1);

   int deerID=rmCreateObjectDef("deer herd");
   bonusChance=rmRandFloat(0, 1);
   if(bonusChance<0.5)
      rmAddObjectDefItem(deerID, "deer", rmRandInt(6,8), 4.0);
   else
      rmAddObjectDefItem(deerID, "deer", rmRandInt(7,9), 6.0);
   rmSetObjectDefMinDistance(deerID, 45.0);
   rmSetObjectDefMaxDistance(deerID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(deerID, avoidDeer);
   rmAddObjectDefConstraint(deerID, avoidAll);
   rmAddObjectDefConstraint(deerID, avoidSocket);
   rmAddObjectDefConstraint(deerID, avoidImpassableLand);
   rmAddObjectDefConstraint(deerID, avoidStartingUnits);
   rmSetObjectDefCreateHerd(deerID, true);
   rmPlaceObjectDefPerPlayer(deerID, false, 4);

   // Text
   rmSetStatusText("",0.80);

   // SHEEP
   int playerSheepID=rmCreateObjectDef("player sheep");
   rmAddObjectDefItem(playerSheepID, "sheep", 2, 4.0);
   rmSetObjectDefMinDistance(playerSheepID, 16.0);
   rmSetObjectDefMaxDistance(playerSheepID, 32.0);
   rmAddObjectDefConstraint(playerSheepID, avoidSheep);
   rmAddObjectDefConstraint(playerSheepID, avoidAll);
   rmAddObjectDefConstraint(playerSheepID, longPlayerConstraint);
   rmAddObjectDefConstraint(playerSheepID, avoidCliffs);
   rmAddObjectDefConstraint(playerSheepID, avoidImpassableLand);
   rmPlaceObjectDefPerPlayer(playerSheepID, false, 2);

   int sheepID=rmCreateObjectDef("sheep");
   rmAddObjectDefItem(sheepID, "sheep", 2, 4.0);
   rmSetObjectDefMinDistance(sheepID, 0.0);
   rmSetObjectDefMaxDistance(sheepID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(sheepID, avoidSheep);
   rmAddObjectDefConstraint(sheepID, avoidAll);
   rmAddObjectDefConstraint(sheepID, longPlayerConstraint);
   rmAddObjectDefConstraint(sheepID, avoidCliffs);
   rmAddObjectDefConstraint(sheepID, avoidImpassableLand);
   rmPlaceObjectDefAtLoc(sheepID, 0, 0.5, 0.5, cNumberNonGaiaPlayers*3);

   int randomTreeID=rmCreateObjectDef("random trees");
   rmAddObjectDefItem(randomTreeID, "TreeNewEngland", 1, 0.0);
   rmSetObjectDefMinDistance(randomTreeID, 25.0);
   rmSetObjectDefMaxDistance(randomTreeID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(randomTreeID, avoidAll);
   rmAddObjectDefConstraint(randomTreeID, avoidTradeRoute);
   rmAddObjectDefConstraint(randomTreeID, avoidSocket);
   rmPlaceObjectDefAtLoc(randomTreeID, 0, 0.5, 0.5, cNumberNonGaiaPlayers*10);

   // Text
   rmSetStatusText("",0.9);

}
